@@ -2,7 +2,7 @@ ezPlot <-
 function (
 	data
 	, dv
-	, sid
+	, wid
 	, within = NULL
 	, between = NULL
 	, between_full = NULL
@@ -19,21 +19,12 @@ function (
 	, y_lab = NULL
 	, split_lab = NULL
 	, levels = NULL
-	, collapse_within = FALSE
+	, diff = NULL
+	, reverse_diff = FALSE
+	, dv_levs = NULL
+	, dv_labs = NULL
+	, row_y_free = FALSE
 ){
-	if(!is.null(levels)){
-		for(i in 1:length(levels)){
-			this_iv = names(levels)[i]
-			data[,names(data)==this_iv] = factor(data[,names(data)==this_iv])
-			if('new_order' %in% names(levels[[i]])){
-				data[,names(data)==this_iv] = factor(data[,names(data)==this_iv],levels=levels[[i]]$new_order)
-			}
-			if('new_names' %in% names(levels[[i]])){
-				levels(data[,names(data)==this_iv]) = levels[[i]]$new_names
-			}
-		}
-	}
-	data = ezStats_main(data,dv,sid,within,between,between_full,collapse_within)
 	if(!(x %in% within) & !(x %in% between) ){
 		stop('"x" not listed in "within" or "between".')
 	}
@@ -72,6 +63,82 @@ function (
 			}
 		}
 	}
+	if(is.data.frame(data)){
+		if(!is.null(levels)){
+			for(i in 1:length(levels)){
+				this_iv = names(levels)[i]
+				data[,names(data)==this_iv] = factor(data[,names(data)==this_iv])
+				if('new_order' %in% names(levels[[i]])){
+					data[,names(data)==this_iv] = factor(data[,names(data)==this_iv],levels=levels[[i]]$new_order)
+				}
+				if('new_names' %in% names(levels[[i]])){
+					levels(data[,names(data)==this_iv]) = levels[[i]]$new_names
+				}
+			}
+		}
+	}else{
+		for(j in 1:length(data)){
+			if(!is.null(levels)){
+				for(i in 1:length(levels)){
+					this_iv = names(levels)[i]
+					data[[j]][,names(data[[j]])==this_iv] = factor(data[[j]][,names(data[[j]])==this_iv])
+					if('new_order' %in% names(levels[[i]])){
+						data[[j]][,names(data[[j]])==this_iv] = factor(data[[j]][,names(data[[j]])==this_iv],levels=levels[[i]]$new_order)
+					}
+					if('new_names' %in% names(levels[[i]])){
+						levels(data[[j]][,names(data[[j]])==this_iv]) = levels[[i]]$new_names
+					}
+				}
+			}
+		}
+	}
+	if(length(dv)==1){
+		if(!is.data.frame(data)){
+			stop('"data" cannot be a list when specifying only one dv.')
+		}
+		data = ezStats(data,dv,wid,within,between,between_full,diff,reverse_diff)
+	}else{
+		if(!is.null(row)){
+			stop('You may not specify a variable to "row" when also specifying multiple dvs.')
+		}
+		if(is.data.frame(data)|(length(dv)!=length(data))){		
+			stop('When specifying multiple dvs, you must provide a list to "data" with as many elements as there are dvs..')
+		}
+		row = .(dv)
+		data_combined = NULL
+		for(this_dv in 1:length(dv)){
+			this_dot_dv = structure(as.list(c(dv[this_dv])),class = 'quoted')
+			if(!is.data.frame(data)){
+				data_combined = rbind(
+					data_combined
+					, cbind(
+						ezStats(data[[this_dv]],this_dot_dv,wid,within,between,between_full,diff,reverse_diff)
+						, dv = as.character(dv[this_dv])
+					)
+				)
+			}else{
+				data_combined = rbind(
+					data_combined
+					, cbind(
+						ezStats(data,this_dot_dv,wid,within,between,between_full,diff,reverse_diff)
+						, dv = as.character(dv[this_dv])
+					)
+				)
+			}
+		}
+		data = data_combined
+		if(!is.null(dv_levs)){
+			if(!is.null(dv_labs)){
+				data$dv = factor(data$dv, levels = dv_levs, labels = dv_labs)
+			}else{
+				data$dv = factor(data$dv, levels = dv_levs)				
+			}
+		}else{
+			 if(!is.null(dv_labs)){
+				levels(data$dv) = dv_labs
+			}
+		}
+	}
 	if(is.null(bar_size)){
 		bar_size = data$FLSD
 	}
@@ -114,10 +181,11 @@ function (
 		p = p+geom_point(
 			aes(
 				colour = split
-				,shape = split
+				, shape = split
 			)
+			, alpha = .8
 		)
-		p = p+opts(legend.background = theme_rect(colour='transparent',fill='transparent'))
+		#p = p+opts(legend.background = theme_rect(colour='transparent',fill='transparent')) #used to be necessary
 		if(!is.null(split_lab)){
 			p = p+labs(colour = split_lab,shape = split_lab)
 		}
@@ -128,6 +196,7 @@ function (
 					,linetype = split
 					,x = as.numeric(x)
 				)
+				, alpha = .8
 			)
 			if(!is.null(split_lab)){
 				p = p+labs(linetype = split_lab)
@@ -143,6 +212,7 @@ function (
 				,linetype = 1
 				,legend = FALSE
 				,width = bar_width
+				, alpha = .5
 			)
 		}
 	}else{
@@ -159,14 +229,23 @@ function (
 				,linetype = 1
 				,legend = FALSE
 				,width = bar_width
+				,alpha = .5
 			)
 		}
 	}
 	if(!is.null(row)){
 		if(!is.null(col)){
-			p = p+facet_grid(row~col)
+			if(row_y_free){
+				p = p+facet_grid(row~col,scales='free_y')
+			}else{
+				p = p+facet_grid(row~col)
+			}
 		}else{
-			p = p+facet_grid(row~.)
+			if(row_y_free){
+				p = p+facet_grid(row~.,scales='free_y')
+			}else{
+				p = p+facet_grid(row~.)
+			}
 		}
 	}else{
 		if(!is.null(col)){
